#ifndef _PRG4_PRGIMG_
#define _PRG4_PRGIMG_

//pre-definition
struct prgimg_t;
/** definition of the prgimg type */
typedef struct prgimg_t prgimg;

/**
 * create an empty image of a given size
 * @param width the width of the new image to create
 * @param height the height of the new image to create
 * @return a pointer to the newly created image, or NULL if a problem occured
 */
prgimg* prgimg_create_empty(int width, int height);
/**
 * read an image from a file
 * @param file_name the path of the file containing the image to read
 * @return the image read, or NULL if a problem occured
 */
prgimg* prgimg_read_file(const char* file_name);


/**
 * Retrieve the red value of a given pixel of the image
 * @param img the image that will be used as source
 * @param x the x coordinate of the pixel
 * @param y the y coordinate of the pixel
 * @return the red value of the pixel at (@a x, @a y)
 */
unsigned char prgimg_get_red(const prgimg* img, int x, int y);
/**
 * Retrieve the green value of a given pixel of the image
 * @param img the image that will be used as source
 * @param x the x coordinate of the pixel
 * @param y the y coordinate of the pixel
 * @return the green value of the pixel at (@a x, @a y)
 */
unsigned char prgimg_get_green(const prgimg* img, int x, int y);
/**
 * Retrieve the blue value of a given pixel of the image
 * @param img the image that will be used as source
 * @param x the x coordinate of the pixel
 * @param y the y coordinate of the pixel
 * @return the blue value of the pixel at (@a x, @a y)
 */
unsigned char prgimg_get_blue(const prgimg* img, int x, int y);
/**
 * Retrieve the height of the image
 * @param img the image we want its height
 * @return the height in pixel of the image
 */
int prgimg_get_height(const prgimg* img);
/**
 * Retrieve the width of the image
 * @param img the image we want its width
 * @return the width in pixel of the image
 */
int prgimg_get_width(const prgimg* img);

/**
 * Set the red value for a given pixel in an image
 * @param img the image
 * @param x the x coordinate of the pixel
 * @param y the y coordinate of the pixel
 * @param red the new red value of the image
 */
void prgimg_set_red(prgimg* img, int x, int y, unsigned char red);
/**
 * Set the green value for a given pixel in an image
 * @param img the image
 * @param x the x coordinate of the pixel
 * @param y the y coordinate of the pixel
 * @param green the new green value of the image
 */
void prgimg_set_green(prgimg* img, int x, int y, unsigned char green);
/**
 * Set the blue value for a given pixel in an image
 * @param img the image
 * @param x the x coordinate of the pixel
 * @param y the y coordinate of the pixel
 * @param blue the new blue value of the image
 */
void prgimg_set_blue(prgimg* img, int x, int y, unsigned char blue);

/**
 * Write the given image on a file
 * @param file_name the path to the file to write the image to
 * @param img the image to write
 */
void prgimg_write_file(const char* file_name, prgimg* img);

/**
 * Properly destroy the memory related to an image
 * @param img the image to destroy
 */
void prgimg_free(prgimg** img);

#endif /* _PRG4_PRGIMG_ */

